@@ -76,7 +76,7 @@ scram_parse_client_first (const char *str, size_t len,
 			  struct scram_client_first *cf)
 {
   /* Minimum client first string is 'n,,n=a,r=b'. */
-  if (strlen (str) < 10)
+  if (strnlen (str, len) < 10)
     return -1;
 
   if (len == 0 || *str != 'n')
@@ -199,7 +199,7 @@ scram_parse_server_first (const char *str, size_t len,
 			  struct scram_server_first *sf)
 {
   /* Minimum server first string is 'r=ab,s=biws,i=1'. */
-  if (strlen (str) < 15)
+  if (strnlen (str, len) < 15)
     return -1;
 
   if (len == 0 || *str != 'r')
@@ -308,7 +308,7 @@ scram_parse_client_final (const char *str, size_t len,
 			  struct scram_client_final *cl)
 {
   /* Minimum client final string is 'c=biws,r=ab,p=ab=='. */
-  if (strlen (str) < 18)
+  if (strnlen (str, len) < 18)
     return -1;
 
   if (len == 0 || *str != 'c')
@@ -413,7 +413,7 @@ scram_parse_server_final (const char *str, size_t len,
 			  struct scram_server_final *sl)
 {
   /* Minimum client final string is 'v=ab=='. */
-  if (strlen (str) < 6)
+  if (strnlen (str, len) < 6)
     return -1;
 
   if (len == 0 || *str != 'v')
