@@ -1,5 +1,5 @@
 /* property.c --- Callback property handling.
- * Copyright (C) 2004, 2005, 2008  Simon Josefsson
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -78,6 +78,27 @@ map (Gsasl_session * sctx, Gsasl_property prop)
 
     case GSASL_DIGEST_MD5_HASHED_PASSWORD:
       p = &sctx->digest_md5_hashed_password;
+      break;
+
+    case GSASL_QOPS:
+      p = &sctx->qops;
+      break;
+
+    case GSASL_QOP:
+      p = &sctx->qop;
+      break;
+
+    case GSASL_SCRAM_ITER:
+      p = &sctx->scram_iter;
+      break;
+
+    case GSASL_SCRAM_SALT:
+      p = &sctx->scram_salt;
+      break;
+
+    case GSASL_SCRAM_SALTED_PASSWORD:
+      p = &sctx->scram_salted_password;
+      break;
 
     default:
       break;
@@ -135,8 +156,7 @@ gsasl_property_set_raw (Gsasl_session * sctx, Gsasl_property prop,
 
   if (p)
     {
-      if (*p)
-	free (*p);
+      free (*p);
       if (data)
 	{
 	  *p = malloc (len + 1);
@@ -181,12 +201,6 @@ gsasl_property_fast (Gsasl_session * sctx, Gsasl_property prop)
   return NULL;
 }
 
-#ifndef GSASL_NO_OBSOLETE
-/* Declared in obsolete.c. */
-const char *
-_gsasl_obsolete_property_map (Gsasl_session * sctx, Gsasl_property prop);
-#endif
-
 /**
  * gsasl_property_get:
  * @sctx: session handle.
