@@ -52,6 +52,26 @@ server_cb_retrieve (Gsasl_session_ctx * xctx,
   return GSASL_OK;
 }
 
+static Gsasl_qop
+server_cb_qop (Gsasl_session_ctx * xctx)
+{
+  int i = *(int*) gsasl_appinfo_get (xctx);
+  if (i == 1 || i == 3)
+    return GSASL_QOP_AUTH;
+  else if (i == 2)
+    return GSASL_QOP_AUTH | GSASL_QOP_AUTH_INT;
+  else
+    return 0;
+}
+
+static Gsasl_qop
+client_cb_qop (Gsasl_session * sctx, Gsasl_qop serverqops)
+{
+  if (serverqops & GSASL_QOP_AUTH_INT)
+    return GSASL_QOP_AUTH_INT;
+  return GSASL_QOP_AUTH;
+}
+
 static int
 client_callback_service (Gsasl_session_ctx * ctx,
 			 char *srv,
@@ -110,15 +130,18 @@ doit (void)
       return;
     }
 
-  if (!gsasl_client_support_p (ctx, "DIGEST-MD5") ||
-      !gsasl_server_support_p (ctx, "DIGEST-MD5"))
+  if (!gsasl_client_support_p (ctx, "DIGEST-MD5")
+      || !gsasl_server_support_p (ctx, "DIGEST-MD5"))
     {
-      printf ("No support for DIGEST-MD5...\n");
-      return;
+      gsasl_done (ctx);
+      fail("No support for DIGEST-MD5.\n");
+      exit(77);
     }
 
   gsasl_server_callback_retrieve_set (ctx, server_cb_retrieve);
 
+  gsasl_server_callback_qop_set (ctx, server_cb_qop);
+
   gsasl_client_callback_service_set (ctx, client_callback_service);
 
   gsasl_client_callback_authentication_id_set (ctx,
@@ -128,6 +151,9 @@ doit (void)
 
   for (i = 0; i < 5; i++)
     {
+      if (i > 2)
+	gsasl_client_callback_qop_set (ctx, client_cb_qop);
+
       res = gsasl_server_start (ctx, "DIGEST-MD5", &server);
       if (res != GSASL_OK)
 	{
@@ -141,6 +167,8 @@ doit (void)
 	  return;
 	}
 
+      gsasl_appinfo_set (server, (void*) &i);
+
       /* Server begins... */
 
       res = gsasl_step (server, NULL, 0, &s1, &s1len);
